using System;
using System.Collections.Generic;
using System.Windows.Forms;
using Infragistics.Win;
using Infragistics.Win.UltraWinDataSource;
using Infragistics.Win.UltraWinGrid;
using Infragistics.Win.UltraWinMaskedEdit;
using ColumnHeader=Infragistics.Win.UltraWinGrid.ColumnHeader;

namespace EditAsTime
{
    public partial class Form1 : Form
    {
        private readonly List<DateTime> data = new List<DateTime>();

        /// <summary>
        /// Initializes a new instance of the <see cref="Form1"/> class.
        /// </summary>
        public Form1()
        {
            InitializeComponent();
            UltraGridColumn startTimeCol = grid_.DisplayLayout.Bands[0].Columns["Start"];
            UltraGridColumn endTimeCol = grid_.DisplayLayout.Bands[0].Columns["End"];

            startTimeCol.UseEditorMaskSettings = true;
            endTimeCol.UseEditorMaskSettings = true;

            ultraMaskedEdit1.EditAs = EditAsType.Time;

            startTimeCol.EditorControl = ultraMaskedEdit1;
            endTimeCol.EditorControl = ultraMaskedEdit1;

            grid_.DisplayLayout.Override.CellClickAction = CellClickAction.RowSelect;
            startTimeCol.CellClickAction = CellClickAction.RowSelect;
            endTimeCol.CellClickAction = CellClickAction.RowSelect;

            BuildTestData();
            PopulateDataSource();
        }

        /// <summary>
        /// Builds the test data.
        /// </summary>
        private void BuildTestData()
        {
            data.Add(new DateTime(1900, 1, 1, 8, 0, 0));
            data.Add(new DateTime(1922, 1, 1, 12, 0, 0));
            data.Add(new DateTime(1944, 1, 1, 16, 0, 0));
        }

        /// <summary>
        /// Populates the data source.
        /// </summary>
        private void PopulateDataSource()
        {
            int i = 5;
            foreach (DateTime time in data)
            {
                UltraDataRow row = dataSource_.Rows.Add();
                row["Start"] = time;
                row["End"] = time;
                row["Value"] = i++;
            }
        }


        #region Workaround

        /// <summary>
        /// Called when [before enter edit mode].
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.ComponentModel.CancelEventArgs"/> instance containing the event data.</param>
        private void OnBeforeEnterEditMode(object sender, System.ComponentModel.CancelEventArgs e)
        {
            UltraGridCell cell = grid_.ActiveCell;
            if (cell != null && cell.IsDataCell)
            {
                if (cell.Column.Key == "Start" || cell.Column.Key == "End")
                {
                    if (cell.Column.CellClickAction == CellClickAction.RowSelect)
                        e.Cancel = true;
                }
            }
        }

        /// <summary>
        /// Called when [before exit edit mode].
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="Infragistics.Win.UltraWinGrid.BeforeExitEditModeEventArgs"/> instance containing the event data.</param>
        private void OnBeforeExitEditMode(object sender, Infragistics.Win.UltraWinGrid.BeforeExitEditModeEventArgs e)
        {
            UltraGridCell cell = grid_.ActiveCell;
            if (cell != null && cell.IsDataCell)
            {
                if (cell.Column.Key == "Start" || cell.Column.Key == "End")
                {
                    if (cell.Column.CellClickAction == CellClickAction.EditAndSelectText)
                        cell.Column.CellClickAction = CellClickAction.RowSelect;
                }
            }
        }

        /// <summary>
        /// Called when [double click row].
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="Infragistics.Win.UltraWinGrid.DoubleClickRowEventArgs"/> instance containing the event data.</param>
        private void OnDoubleClickRow(object sender, DoubleClickRowEventArgs e)
        {
            UltraGrid grid = (UltraGrid)sender;
            if (grid != null)
            {
                UIElement element = grid.DisplayLayout.UIElement.LastElementEntered;
                if (element != null)
                {
                    UltraGridCell cell = element.GetContext(typeof(UltraGridCell)) as UltraGridCell;
                    if (cell != null && cell.IsDataCell)
                    {
                        if (cell.Column.Key == "Start" || cell.Column.Key == "End")
                        {
                            cell.Activate();
                            cell.Column.CellClickAction = CellClickAction.EditAndSelectText;
                            grid.PerformAction(UltraGridAction.EnterEditMode);
                        }
                    }
                }
            }

        }

        #endregion
    }
}